/* 
 * module.c
 *
 * Routines to access the different modules.
 *
 * Copyright (C) 1997,1998 Hermann Samso
 */

#include <stdlib.h>
#include <stdio.h>
#include "delfin.h"
#include "module.h"

MODULE* firstmodule = NULL;
MODULE* actualmodule = NULL;
INT nr_modules = 0;

void ModuleRegister(MODULE* mod) {
	MODULE* cruise = firstmodule;


	if (trace_on)
		fprintf(stderr, "ModuleRegister()\n");
		

	if (mod == NULL)
		return;
					
	if (cruise != NULL) {
		while (cruise->next != NULL)
			cruise = cruise->next;
		cruise->next = mod;
		mod->previous = cruise;
	}
	else {
		firstmodule = mod;
		firstmodule->previous = NULL;
	}
	nr_modules++;
	actualmodule = mod;
	ModuleInit();
}


void ModuleRegisterAll(void) {
	if (trace_on)
		fprintf(stderr, "ModuleRegisterAll()\n");


	ModuleRegister(&wave_mod);
	ModuleRegister(&actualizer_mod);
	ModuleRegister(&vol_mod);
	ModuleRegister(&noise_mod);
	ModuleRegister(&echo_mod);
	ModuleRegister(&mixer_mod);
	actualmodule = firstmodule;
}


INT ModuleHowMany(void) {
	if (trace_on)
		fprintf(stderr, "ModuleHowMany()\n");
		
	return(nr_modules);
}



BOOL ModuleHasMore(void) {
	if (trace_on)
		fprintf(stderr, "ModuleHasMore()\n");
		
	if (actualmodule->next != NULL)
		return(TRUE);
	else
		return(FALSE);
}


void ModuleFirst(void) {
//	if (trace_on)
//		fprintf(stderr, "ModuleFirst()\n");
		
	if (firstmodule == NULL)
		return;
	else
		actualmodule = firstmodule;
}

	
void ModuleNext(void) {
	if (trace_on)
		fprintf(stderr, "ModuleNext()\n");

	if (actualmodule != NULL)
		if (actualmodule->next != NULL)
			actualmodule = actualmodule->next;
}


void ModulePrevious(void) {
	if (trace_on)
		fprintf(stderr, "ModulePrevious()\n");
		
	if (actualmodule != NULL)
		if (actualmodule->previous != NULL)
			actualmodule = actualmodule->previous;
}


CHAR* ModuleGetName(void) {
//	if (trace_on)
//		fprintf(stderr, "ModuleGetName()\n");
		
	if (actualmodule == NULL)
		return(NULL);
	return(actualmodule->name);
}


void ModuleSelectByName(CHAR* name) {
	MODULE* cruise = firstmodule;


//	if (trace_on)
//		fprintf(stderr, "ModuleSelectByName(%s)\n", name);
		

	while (cruise != NULL) {
		if (strcmp(name, cruise->name) == 0)
			actualmodule = cruise;
		cruise = cruise->next;
	}
}


void ModuleInit(void) {
	if (trace_on)
		fprintf(stderr, "ModuleInit()\n");


	if (actualmodule != NULL)
		actualmodule->init();
}


void ModuleExit(void) {
	if (trace_on)
		fprintf(stderr, "ModuleExit()\n");


	if (actualmodule != NULL)
		actualmodule->exit();
}


void ModuleCtrl(INT input) {
	if (trace_on)
		fprintf(stderr, "ModuleCtrl()\n");


	if (actualmodule != NULL)
		actualmodule->ctrl(input);
}


void ModuleExec(void) {
	if (trace_on)
		fprintf(stderr, "ModuleExec()\n");
	
	
	if (actualmodule != NULL)
		actualmodule->exec();
}


void ModuleDisplay(void) {
	if (trace_on)
		fprintf(stderr, "ModuleDisplay()\n");
	        
	        
	if (actualmodule != NULL)
		actualmodule->display();
}
